@@ -81,6 +81,8 @@
 #' @param L Number of folds for cross-fitting nuisance models.
 #' @param cross_fit_g_models If \code{TRUE}, the g-models will not be
 #' cross-fitted even if L > 1.
+#' @param cross_fit_c_models If \code{TRUE}, the c-models will not be
+#' cross-fitted even if L > 1.
 #' @param save_cross_fit_models If \code{TRUE}, the cross-fitted
 #' models will be saved.
 #' @param future_args Arguments passed to [future.apply::future_apply()].
@@ -168,13 +170,14 @@
 #' po$qv_functions$stage_1
 #' head(get_policy(pe)(pd))
 #' @export
-policy_learn <- function(type = "ql",
-                         control = list(),
+policy_learn <- function(type = "blip",
+                         control = control_blip(),
                          alpha = 0,
                          threshold = NULL,
                          full_history = FALSE,
                          L = 1,
                          cross_fit_g_models = TRUE,
+                         cross_fit_c_models = TRUE,
                          save_cross_fit_models = FALSE,
                          future_args = list(future.seed = TRUE),
                          name = type) {
@@ -195,7 +198,7 @@ policy_learn <- function(type = "ql",
     }
     threshold <- unique(sort(threshold))
   }
-  if (threshold_indicator == FALSE) {
+  if (isFALSE(threshold_indicator)) {
     threshold <- 0
   }
   if (!(is.logical(full_history) && (length(full_history) == 1))) {
@@ -236,6 +239,7 @@ policy_learn <- function(type = "ql",
     threshold = threshold,
     L = L,
     cross_fit_g_models = cross_fit_g_models,
+    cross_fit_c_models = cross_fit_c_models,
     save_cross_fit_models = save_cross_fit_models,
     future_args = future_args,
     full_history = full_history
@@ -299,7 +303,10 @@ policy_learn <- function(type = "ql",
 pl <- function(call, args) {
   function(policy_data,
            g_models = NULL, g_functions = NULL, g_full_history = FALSE,
-           q_models, q_full_history = FALSE) {
+           q_models, q_full_history = FALSE,
+           c_models = NULL, c_functions = NULL, c_full_history = FALSE,
+           m_model = NULL, m_function = NULL, m_full_history = FALSE) {
+    ## evaluating the learner:
     eval_args <- as.list(environment())
     args <- append(args, eval_args)
     do.call(what = call, args)
